;;;;	Project: SN
;;;;
;;;;	Module: host.sc
;;;;
;;;;	Author: Jeff Stephenson
;;;;
;;;;	The Host class is SCI's interface to the host machine.


(module#	HOST)




(class Host of Object
	(properties
		int	notify	0					;Who to notify upon success
		id		from		0					;Sender of this message
		id		to			0					;Receiver of this message
	)

	(methods
		lookup
		newObj
		send
		rSend
		sendArray
		rSendArray
		lock
		lockQueue
		unlock
		getObj
		add
		setEq
		setNe
		login
		logout
		register
	)



	(method (init maxSID)
		(SID SIDINIT maxSID)
		(super init:)
	)


	(method (lookup s)
		(return (SID SIDLOOKUP s))
	)


	(method (send)
	;; Send a message.
		(TSN TSN_SEND
			"w+"
			(| notify SN_SEND)
			(to sid?)
			(if from (from sid?) else 0)
			&rest
		)
	)


	(method (rSend)
	;; Send a message recursively.
		(TSN TSN_SEND
			"w+"
			(| notify SN_RSEND)
			(to sid?)
			(if from (from sid) else 0)
			&rest
		)
	)


	(method (sendArray type len msg)
	;; Send the array 'msg'.  'len' is the number of bytes in the array.
		(TSN TSN_SEND
			"wwwwa"
			(| notify SN_SEND)
			(to sid?)
			(if from (from sid) else 0)
			type
			len
			(msg data?)
		)
	)


	(method (rSendArray type len msg)
	;; Send the array 'msg'.  'len' is the number of bytes in the array.
		(TSN TSN_SEND
			"wwwwa"
			(| notify SN_RSEND)
			(to sid?)
			(from sid?)
			(if from (from sid) else 0)
			type
			len
			(msg data?)
		)
	)


	(method (lock num array)
	;; Lock objects.
		(if (== argc 1)
			(TSN TSN_SEND
				"w+"
				(| SN_LOCK notify)
				(from sid?)
				num
			)
		else
			(TSN TSN_SEND
				"wwa"
				(| SN_LOCK notify)
				(from sid?)
				num
				(array data?)
			)
		)
	)


	(method (lockQueue)
	;; Lock an object.  If unable to do so, add the sender to a
	;; queue waiting for a lock on the object.
		(TSN TSN_SEND
			"w+"
			(| notify SN_LOCKQUEUE)
			(to sid?)
			(from sid?)
		)
	)


	(method (unlock num array)
	;; Unlock objects.
		(if (== argc 1)
			(TSN TSN_SEND
				"w+"
				(| SN_UNLOCK notify)
				(from sid?)
				num
			)
		else
			(TSN TSN_SEND
				"wwa"
				(| SN_UNLOCK notify)
				(from sid?)
				num
				(array data?)
			)
		)
	)


	(method (getObj)
	;; Request a dump of the properties of the receiver.
		(TSN TSN_SEND
			"w+"
			(| notify SN_GETOBJ)
			(to sid?)
			(from sid?)
		)
	)


	(method (add what value)
	;; Add 'value' to the property 'what' of the receiver.
		(TSN TSN_SEND
			"w+"
			(| notify SN_ADD)
			(to sid?)
			(from sid?)
			(ObjPropOffset to what)
			value
		)
	)


	(method (setEq what testVal newVal)
	;; Set the property 'what' to 'newVal' if it equals 'testVal'.
		(TSN TSN_SEND
			"w+"
			(| notify SN_SETEQ)
			(to sid?)
			(from sid?)
			(ObjPropOffset what)
			testVal
			newVal
		)
	)


	(method (setNe what testVal newVal)
	;; Set the property 'what' to 'newVal' if it is not equal to 'testVal'.
		(TSN TSN_SEND
			"w+"
			(| notify SN_SETNE)
			(to sid?)
			(from sid?)
			(ObjPropOffset what)
			testVal
			newVal
		)
	)


	(method (login password autoPass)
		(if argc
			(TSN TSN_SEND
				"bbwwwsbba"
				SN_LOGIN
				0
				0
				(player userIDLow?)
				(player userIDHigh?)
				(player name?)
				2
				autoPass
				10
				(password data?)
			)
		else
		;; Old format uid login
			(TSN TSN_SEND
				"wwwws"
				SN_LOGIN
				0
				(player userIDLow?)
				(player userIDHigh?)
				(player name?)
			)
		)
	)


	(method (logout)
		(TSN TSN_DISCONNECT)
	)


	(method (register)
		(TSN TSN_SEND
			"bw+"
			SN_REGISTER
			&rest
		)
	)
)

