;;;;	Project: SN
;;;;
;;;;	Module: msg.sc
;;;;
;;;;	Author: Jeff Stephenson
;;;;
;;;;	Messages from the host.


(module#	MSG)

(include globals.sh)
(include bbs.sh)
(include email.sh)
(include hints.sh)

(class Msg of String
	(properties
		int	-len-			0
		int	command		0
		int	toSID			0
		id		to				0
		int	claimed		0
	)

	(methods
		wordAt
		stringAt
		arrayAt
		elementAt
	)


	(method (new array len &tmp msg)
		(= msg (Clone self))
		(msg
			data: array,
			-len-: len
		)
		(return msg)
	)


	(method (init)
		(= command (self at: 0))
		(= toSID (self wordAt: 2))
		(= to (Host lookup: toSID))

		(if 
			(and
				(& debugMsgs BRK_ON_MSG)
				(== -dict- dbgMsgType)
			)

			(SetDebug)
		)
	)

	(method (dispose)
		(= sid 0)    ;; just to be sure
		(super dispose: &rest) 
	)

	(method (size)	;*** Use array size: instead of string.
		(return (KArray ArraySize data))
	)

	(method (with array len &tmp msgClass msg)
		(= command (KArray ArrayAt array 0))
		(= msgClass
			(switch command
				(SN_ACK
					AckMsg
				)
				(SN_NAK
					NakMsg
				)
				(SN_SET
					SetMsg
				)
				(SN_SETINT
					SetIntMsg
				)
				(SN_SETSTR
					SetStrMsg
				)
;				(SN_GETOBJ
;					ObjMsg
;				)
				(SN_GRPPROP
					GrpGetProp
				)
				(SN_SEND
					Send
				)
				(SN_OBJID
					ObjID
				)
				(SN_GRPMEM
					GrpMem
				)
				(SN_GRPJOIN
					GrpJoin
				)
				(SN_GRPDEL
					GrpDel
				)
				(SN_OBJFREE
					ObjFree
				)
				(SN_SYSOP_CMD
					SysOpCmd
				)
				(SN_RMI
					RMI
				)
				(SN_BBS
					BBSMsg
				)
				(SN_GAMEVERSION
					GameVersion
				)
				(SN_ACCOUNTING
					AccountMsg
				)
				(SN_HOSTINFO
					HostInfo
				)
				(SN_HINT
					HintMsg
				)
				(SN_EMAIL
					EMMsg
				)
				(SN_HOSTFILE
					HostFile
				)
				(SN_OBJEXISTS
					ObjExists
				)
				(SN_GAMERIGHTS
					GameRights
				)
				(US_REQUEST
					NewBoxHandler
				)
				(SN_WAITGRPREQUEST
				 	WaitGrpRequest
				)
				(SN_UNSOLICITED
				 	Unsolicited
				)
				(SN_USERINFO
					UserInfoMsg
				)
				(SN_USERFIND
					UserFindMsg
				)
				(SN_OBJNEWLIST
					ObjNewList
				)
				(SN_OBJFREELIST
					ObjFreeList
				)
				(else
					0
				)
			)
		)

		(if msgClass
			(= msg (msgClass new: array len))
			(msg init:)
		else
			(cond
				((not (Debug))
					(Print "Unsupported message." 
						#p_button: "__OK__" 0
					)
				)
				((Print
						"Unsupported message."
						#p_button	" Debug "	1
						#p_button	" Ignore "	0
					)
					(SetDebug array)
				)
			)
			(= msg 0)
		)

		(return msg)
	)


	(method (wordAt n)
	;; Extract the word at position n in the byte array.  This assumes
	;; Intel word ordering of the bytes (least-significant byte first).
		(return (+ (self at: n) (<< (self at: (+ n 1)) 8)))
	)


	(method (stringAt n)
		(return (self subStr: n -1))
	)


	(method (arrayAt n)
		(return ((ByteArray new:) copyToFrom: 0 self n (- -len- n)))
	)


	(method (elementAt ctrlStr n &tmp i j)
	;; Using the string 'ctrlStr' in which 'b' stands for a byte, 'w' a
	;; word, 'a' an word array, and 's' a string, extract element number
	;; 'n' from the message and return its value (the ID of a WordArray
	;; or String in the case of 'a' and 's').

		;Move to the appropriate element of the control string, counting
		;the number of bytes.
		(for	((= i 0) (= j 0))
				(< j n)
				((++ j))

			(switch (ctrlStr at: j)
				(`b
					(++ i)
				)
				(`w
					(+= i 2)
				)
				(`a
					(Print "Reached `a too early in control string for 'elementAt'")
					(return NULLID)
				)
				(`s
					(Print "Reached `s too early in control string for 'elementAt'")
					(return NULLID)
				)
			)
		)

		;Return a value based on the current position in the control string
		;and the message.
		(switch (ctrlStr at: n)
			(`b
				(return (self at: i))
			)
			(`w
				(return (self wordAt: i))
			)
			(`s
				(return (self stringAt: i))
			)
			(`a
				(return (self arrayAt: i))
			)
		)
	)
)





(class Send of Msg
	(properties
		int	msgType	0
		id		from		0
		int	fromSID	0
	)

	(method (init)
		(super init:)
		(if (= fromSID (self wordAt: 4))
			(= from (Host lookup: fromSID))
		)	   
		(= msgType (self wordAt: 6))
		(self move: 0 8 -1)
		(-= -len- 8)
	)
)

(class AckMsg of Msg
	(properties
		int	whichCmd		0
		int	whichSub		0
	)

	(method (init)
		(super init:)
		(= whichCmd (self at: 4))
		(= whichSub (self at: 5))
	)
)

(class NakMsg of AckMsg
	(properties
		int	numTries	0
	)

	(method (init)
		(super init:)
		(= numTries (self at: 7))
	)
)

(class ObjID of Msg
	(properties
		int	sid			0
	)

	(method (init)
		(super init:)
		(= toSID 0)
		(= to (self wordAt: 2))
		(= sid (self wordAt: 6))
	)

	(method (dispose)
		(= sid 0)
		(super dispose:)
	)
)

;; list of sids starts at wordAt: 8
(class ObjNewList kindof Msg
	(method (init &tmp theSize)
		(super init:)
		(= to (self wordAt: 4))
		(= theSize (self wordAt: 6))

		(self move: 0 8 -1)
		(-= -len- 8)
	)
)

(class ObjFreeList kindof Msg
	(method (init)
		(super init:)
		(= to (self wordAt: 2))

		(self move: 0 6 -1)
		(-= -len- 6)
	)
)

(class SetIntMsg of Msg
	(properties
		id		from			0
		int	prop			0
		int	value			0
		int	number		0
	)

	(methods
		getProp
	)


	(method (init)
		(super init:)
		(if (IsObject to)
			(self getProp: 0)
			(= number (/ (- -len- 6) 4))
		)
	)


	(method (getProp n &tmp i)
		(= i (+ 6 (* n 4)))
		(= prop (ObjOffsetProp to (self wordAt: i)))
		(= value (self wordAt: (+ i 2)))
	)


	(method (doit &tmp i)
		(if (IsObject to)
			(for	((= i 0))
					(< i number)
					((++ i))

				(self getProp: i)
				(if (to respondsTo: (self prop?))
					(to (self prop?) value)
				)
			)
		)
	)
)



(class SetStrMsg of Msg
	(properties
		id		from			0
		int	prop			0
		id		value			0
	)

	(method (init &tmp whatKind whom msgLen)
		(super init:)
		(= prop (ObjOffsetProp to (self wordAt: 6)))

		;;; We will lookup the current type if it exists, and preserve the type
		(if (and (IsObject to) (to respondsTo: (self prop?)))
			(= whom prop)
			(if (IsObject (to whom?))
				(= whatKind
					(switch ((to whom?) type?)
						(BYTEARRAY	ByteArray)
						(WORDARRAY	IntArray)
						(IDARRAY		IDArray)
						(else			String)
					)
				)
			else
				(= whatKind String)	;;; no pre-existing so just make it a String
			)

			(= msgLen (- -len- 8))
			(if (!= whatKind String)	;;; get rid of null that is appended because setStr.
				(-- msgLen)
			)
			(= value 
				((whatKind new:)
					copyToFrom: 0 self 8 msgLen
				)
			)
		)
	)


	(method (doit &tmp whom)
		(if (and (IsObject to) (to respondsTo: (self prop?)))
			(= whom prop)
			(if (IsObject (to whom))
				((to whom) dispose:)
			)
			(to whom value)
		)
	)


	(method (dispose)
		(if
			(and
				(IsObject value)
				(or
					(not (IsObject to))
					(not (to respondsTo: (self prop?)))
					(!= (to (self prop?)) value)
				)
			)
			(value dispose:)
		)
		(super dispose:)
	)
)



(class GrpMem of Msg
	(properties
		id		members		0
		int	number		0
	)

	(method (init)
		(super init:)
		(-= -len- 6)
		(= members
			((IntArray new:)
				copyToFrom: 0 self 6 -len-
			)
		)
		(= number (/ -len- 2))
	)
)



(class GrpJoin of Msg
	(properties
		int	who			0
	)


	(method (init)
		(super init:)
		(= who (self wordAt: 4))
	)
)



(class GrpDel of Msg
	(properties
		int	who			0
	)


	(method (init)
		(super init:)
		(= who (self wordAt: 4))
	)
)



(class ObjFree of Msg
)



(class SysOpCmd of Msg
	(properties
		int	msgType	0
	)

	(method (init)
		(= command (self at: 0))
		(= msgType (self at: 1))
		(= to player)
		(self move: 0 2 -1)
		(-= -len- 2)
	)
)



(class RMI of Msg
	(properties
		int	number	0				;method selector number
	)

	(method (init)
		(super init:)
		(= number (self wordAt: 4))
		(self move: 0 6 -1)
		(-= -len- 6)
	)


	(method (doit)
		(if to
			(InvokeMethod to number (/ -len- 2) data)
		)
	)
)

;** Class handle sysop messages to download billing
;** info from the Host
(class AccountMsg of Msg
	(properties
		int	whichCmd	0
		int	len		0
		id		from		0
	)

	;** called when an I am received
	(method (init)
		(super init:)
		(= whichCmd (self at: 1))
	)

	; ** dispose of all possible memory I have
	(method (dispose)
		(if (IsObject from) 
			(from dispose:) 
		)
		(super dispose:)
	)
)

;; Just handle getting the BB message --
;; bulletin.sc will parse it out
(class BBSMsg of Msg
	(properties
		int	whichCmd		0	;; Which BB command
		int	number		0	;; Bulletin board number
		int	type			0	;; Bulletin board type (purpose)
	)

	(method (init)
		(self
			command:		(self at: 0),
			whichCmd:	(self at: 1),
			number:		(self wordAt: 2),
			toSID:		(self wordAt: 4),
			type:			(self wordAt: 6),
			to:			(Host lookup: (self wordAt: 4)),
		)
		
	;; Maintain backward compatibility w/backward conference rooms
		(if
			(and
				(== type BBS_TYPE_CONFERENCE)
				(== whichCmd BBS_READ)
			)
			(self move: 0 18 -1)
			(if (& prvlgLevel (| PRVLG_SYSOP PRVLG_SUPER))
				(self move: 0 4 -1)
			)
		)
		(if 
			(and
				(& debugMsgs BRK_ON_MSG)
				(== -dict- dbgMsgType)
			)
			(SetDebug)
		)
	)
)


(class GameVersion of Msg
	(properties
		int	number		0
		int	majorVer		0
		int	minorVer		0
	)


	(method (init)
		(= command (self at: 0))
		(= toSID (self wordAt: 1))
		(= to (Host lookup: toSID))
		(= number (self at: 3))
		(= majorVer (self at: 4))
		(= minorVer (self at: 5))

		(if 
			(and
				(& debugMsgs BRK_ON_MSG)
				(== -dict- dbgMsgType)
			)

			(SetDebug)
		)
	)
)




(class SetMsg of Msg
	(properties
		int	prop		0
		int	value		0
		int	type		0
	)

	(methods
		getProp
	)


	(method (doit)
		(self move: 0 6 -1)
		(-= -len- 6)
		(while -len-
			(self getProp:)
			(if (and (IsObject to) (to respondsTo: (self prop?)))
				(to (self prop?) value)
			)
		)
	)


	(enum
		PROP_INT
		PROP_STR
	)


	(method (getProp &tmp i whatKind whom)
		(= type (self at: 0))
		(switch type
			(PROP_INT
				(= prop (ObjOffsetProp to (self wordAt: 1)))
				(= value (self wordAt: 3))
				(self move: 0 5 -1)
				(-= -len- 5)
			)
			(PROP_STR
				(= prop (ObjOffsetProp to (self wordAt: 1)))
				(= whom prop)
				(self move: 0 3 -1)
				(= i (+ (self indexOf: 0) 1))
				(if (and (IsObject (to whom)) ((to whom) respondsTo: #type))
					(= whatKind
						(switch ((to whom) type?)
							(BYTEARRAY	ByteArray)
							(WORDARRAY	IntArray)
							(IDARRAY		IDArray)
							(else			String)
						)
					)
				else
					(= whatKind String)	;;; no pre-existing so just make it a String
				)
				(= value ((whatKind new:) copyToFrom: 0 self 0 i))
				(self move: 0 i -1)
				(-= -len- (+ 3 i))
			)
		)
	)
)


(class HostInfo of Msg
	(properties
		id		value			0
		int	type			0
	)

	(method (init)
		(= command (self at: 0))
		(= type (self at: 1))
		(= to theGame)
		(= value 
			((String new:)
				copyToFrom: 0 self 2 -1
			)
		)
	)

	(method (dispose)
		(if (IsObject value) 
			(value dispose:)
		)
		(super dispose:)
	)
)

(class EMMsg of Msg
	(properties
		int	whichCmd		0
		int	lowWord		0
		int	highWord		0
		int 	status 		0
	)
	(method (init)
		(super init: &rest)
		(= whichCmd (self at: 1))
		(= lowWord (self wordAt: 4))
		(= highWord (self wordAt: 6))
		(= status (self wordAt: 8))
	)
)

(class HintMsg of Msg
	(properties
		int	whichCmd		0
		int	which			0
		id		string		NULLID
	)

	(method (init)
		(= command (self at: 0))
		(= whichCmd (self at: 1))
		(= toSID (self wordAt: 2))	
		(= to (Host lookup: toSID))
		(= which (self at: 4))
		((= string (String new:))
			copyToFrom: 0 self 5 (- -len- 6),
			at: (- -len- 5) 0,
		)
		(if 
			(and
				(& debugMsgs BRK_ON_MSG)
				(== -dict- dbgMsgType)
			)
			(SetDebug self)
		)
	)
	(method (dispose)
	 	(if (IsObject string)
		 	(string dispose:)
		)
		(super dispose: &rest)
	)
)

(class HostFile of Msg
	(properties
		int	whichCmd		0
	)

	(method (init)
		(= command (self at: 0))
		(= whichCmd (self at: 1))
		(self move: 0 2 -1)
		(= to theGame)
	)
)

(class ObjExists of Msg
	(properties
		int	sid			0
	)
	(method (init)
		(super init:)
		(self move: 0 6 -1)
		(-= -len- 6)
	)
)

(class GameRights of Msg
	(method (init)
		(super init:)
		(self move: 0 4 -1)
		(-= -len- 4)
	)
)

(class NewBoxHandler of Msg
	(properties
		int	whichCmd 0
		int	status 	0
		int	lowWord 		0
		int	highWord		0
	)

	(method (init)
		(super init: &rest)
		(= whichCmd (self at: 1))
		(= status 	(self at: 4))
      (= lowWord (self wordAt: 5))
      (= highWord (self wordAt: 7))
	)
)

(class GrpGetProp of Msg
	(properties
		id  from		 0
		int whichCmd 0
	)
	(method (init
					&tmp
						i j k str who index count
						numMembers numProps prp
						[propOffset 20] [propType 20]
			  )
		(super init:)
		(= from		  (Host lookup: (self wordAt: 4)))	
		(= numMembers (self wordAt: 6))
		(= numProps   (self wordAt: 8))
		(for ((= i 10) (= j 0)) (< i (+ 10 (* numProps 4))) ((+= i 4) (++ j))
		 	(= [propOffset j] (self wordAt: i))
			(= [propType   j] (self wordAt: (+ i 2)))
		)
		(= index (+ 10 (* numProps 4)))
		(= count 0)
		(while (and (< count numMembers) (< index (self -len-)))
			(= who (Host lookup: (self wordAt: index)))
			(+= index 2)
			(for ((= j 0)) (< j numProps) ((++ j))
				(if (IsObject who)
					(= prp (ObjOffsetProp who [propOffset j]))
				else
					(= prp -1)
				)
				(if (== [propType j] 1) ;; if a string
					;;; Find where the end of string is...
					(= k index)
					(while (and (< k (self -len-)) (self at: k))
						(++ k)
					)
					;; include the null in str
					(++ k) 

					;;; Copy str from index to null
					((= str (String new: 0))
						copyToFrom: 0 self index (- k index)
					)
					;;; Assign the string to the given property 
					(if (and (IsObject who) (!= who player) (who respondsTo: prp))
						(if (IsObject (who prp?))
							((who prp?) dispose:)
						)
						(who prp str)
					)
					(= index k)
				else						 ;; else must be a value
					(if (and (IsObject who) (!= who player) (who respondsTo: prp))
						(who prp (self wordAt: index))
					)
					(+= index 2)
				)
			)
			(++ count)
		)
	)
)

(class WaitGrpRequest of Msg
	(properties
		id		from			0
		int   msgType		0	;; (0 = Land description, 1 = Land info)
	)
	(method (init)
		(= command 	  (self at: 0))
		(= msgType	  (self at: 1))	
		(= toSID 	  (self wordAt: 2))
		(= to 		  (Host lookup: toSID))
		(= from		  (Host lookup: (self wordAt: 4)))
			
		(self move: 0 6 (- (self -len-) 6))
	)
)

(class Unsolicited of Msg
	(method (init)
		(= command	(self at: 0))
		(= toSID		0)
		(= to 		(Host lookup: 0))

		(self move: 0 2 (- (self -len-) 2))
	)
)

(class UserInfoMsg of Msg
	(properties
		int	whichCmd	0
		int	value		0
		int	msgType	0
		int	whichSub	0
	)
	(method (init)
		(= command (self at: 0))
		(= whichCmd (self at: 1))
		(= msgType (self wordAt: 4))
		(= whichSub (self wordAt: 6))
		(switch msgType
			(1
				(= value (LongInt new: (self wordAt: 10) (self wordAt: 8)))
			)
		)

		(= toSID		0)
		(= to 		(Host lookup: 0))
	)
	(method (dispose:)
		(super dispose:)
		(if (IsObject value)
			(value dispose:)
		)
	)
)


(class UserFindMsg of Msg
	(method (init)
		(super init:)
		(self move: 0 4 -1)
		(-= -len- 4)
	)
)

