(module# MSGPROC)

(define CHAT 			1)

(include globals.sh)
(include tsn.sh)

(class NetMsgProcessor of Code
   (properties
	   id nmpSystemQueue 0		;; system message queue **highest priority**
		id nmpGameQueue   0     ;; game message queue **high priority**
		id nmpChatQueue   0     ;; chat message queue **lowest prioirty**

		id lastProcessor 0   ;; last NetMsgProcessor
		id lastMsg		  0   ;; last message that was processed
		
		id	passSystemTo	0	;; ID of object to pass system messages to
		id passChatTo		0 	;; ID of object to pass chat messages to
		id passGameTo 0		;; ID of object to pass game messages to
	)

	(methods
	   init
		dispose

		getMsg		;; retrieve a message from a queue
		putMsg		;; put a message in a queue

		handleMsg	;; process a message from a given queue
		handleEvent ;; takes a remote event messages and stuffs them into appropriate queues

		doit			;; process the queues or stuff them if necessary
	)

	(method (init)
	   (super init:)

		;; if the global NetMessageProcessor is not this NetMessageProcessor
		;; then attach this one.
		(if (!= netMsgProcessor self)
			(= lastProcessor netMsgProcessor)

		   ;; attach myself to the global netMsgProcessor and inDialogHandler
			;; variables
		   (= netMsgProcessor (= inDialogHandler self))

			(= nmpSystemQueue (List new:))
;			(= nmpGameQueue   (List new:))
			(= nmpGameQueue nmpSystemQueue)
			(= nmpChatQueue   (List new:))
		)
	)

	(method (dispose)
	   (= netMsgProcessor (= inDialogHandler lastProcessor))

		(nmpSystemQueue dispose:)
;		(nmpGameQueue   dispose:)
		(nmpChatQueue   dispose:)

		(super dispose:)
	)

	;** method getMsg: will return the message you request
	(method (getMsg whichQueue whichMsg &tmp msg)
	   ;; get the passed message from the passed queue unless there is no 
		;; passed message number, then get message number zero.
	   (= msg (whichQueue at: (if (> argc 1) whichMsg else 0)))

		;; delete the message that we just got, the update the lastMsg 
		(whichQueue delete: (= lastMsg msg))
		(return msg)
	)

	(method (putMsg whichQueue msg) ;; &tmp s oldport)
	   (whichQueue add: msg)

;		(= oldport (GetPort))
;		(SetPort 0)
;
;	   (= s (String format: "Sys = %d  Game = %d  Chat = %d  inDialog = %d" 
;		   (nmpSystemQueue size:) (nmpGameQueue size:) (nmpChatQueue size:) inDialog))
;
;	   (Display s
;		   p_at: 0 0
;		   p_color: white
;			p_back: black
;			p_font: smallFont
;		)
;
;		(SetPort oldport)
;
;		(s dispose:)
	)

	(method (handleMsg whichQueue &tmp msg)
	   ;; process message from the passed message queue
		(= msg (self getMsg: whichQueue &rest))

		(if (not (IsObject msg?))
			(return)
		)

		;; make sure that messages have the needed from property set
		(if 
			(and
				(msg respondsTo: #from)
				(msg respondsTo: #fromSID)
			)

			(msg from: (Host lookup: (msg fromSID?)))
		)

		;;; if this is a send and not a system message and 'from' has left, 
		;;; then get rid of this msg
		(if (and
				(!= whichQueue nmpSystemQueue)
				(msg respondsTo: #fromSID)
				(not (IsObject (Host lookup: (msg fromSID?))))
			)

			(msg dispose:)
			(return)
		)

		(if (IsObject (msg to?))
		   ;; whoever this message is for will handle it 
		   ((msg to?) handleMsg: msg)
		else
		   (if debugOn
		   	(Print "NetMsgProcessor: Msg with illegal to? property.")
			)
		)

		;; only dispose of the message if it is not already in the queues
		(if 
			(and
				(not (nmpGameQueue contains: msg))
				(not (nmpSystemQueue contains: msg))
				(not (nmpChatQueue contains: msg))
			)

			(msg dispose:)
		)

		;; TSN_DOIT and TSN_FLUSH if necessary
		(if (and inDialog (>> (TSN TSN_INFO) 8))
		   (TSN TSN_DOIT)
			(TSN TSN_FLUSH)
		)
	)

	(method (doit &tmp msg obj)
	   (cond
			;; if in stasis state do not process any other messages.
			(inDialog
			   (self handleEvent:)
			)

			((nmpSystemQueue size:)
			   ;; handle all system messages
;		   	(while (nmpSystemQueue size:)
				   (self handleMsg: nmpSystemQueue)
;				)
			)

			;; handle any game messages
			((nmpGameQueue size:)
			   (self handleMsg: nmpGameQueue)
			)

			;; handle any chat messages
			((nmpChatQueue size:)
			   ;; check to make sure that this message has a valid from object
				(= msg (nmpChatQueue at: 0))

				(if (or
						(IsObject (msg from?))
						(and
							(msg fromSID?)
							(= obj (Host lookup: (msg fromSID?)))
							(msg from: obj)
						)
					)
				   (self handleMsg: nmpChatQueue)
				else
				   (nmpChatQueue delete: msg)
					(msg dispose:)
				)
			)
		)
	)

	(method (handleEvent evt &tmp event msg s s1)
	   (if (>> (TSN TSN_INFO) 8)
			(TSN TSN_FLUSH)
			(TSN TSN_DOIT)

		   ;; if argument is passed in, then do a normal handleEvent, otherwise
			;; caller is expecting this function to get the remote event.
			(= event
			   (if (== argc 1)
				   evt
				else
				   (user nextEvent: remoteEvt)
				)
			)

			(if 
			   (and
			      (IsObject event)
					(== (event type?) remoteEvt)
				)

				(= msg (Msg with: (evt message?) (event modifiers?)))

				(if (== msg NULLID)
				   (return FALSE)
				)

				(if (& debugMsgs SHOW_MSG)
				   (Debug DInspect msg)
				)

				(cond
				   ((IsObject (msg to?))
					   (cond
						   ((== (msg command?) SN_SEND)
							   ;; put all chat messages in chat queue (max 40)
								(if (== (msg msgType?) CHAT) 
									(if passChatTo
										(passChatTo handleMsg: msg)
									else
									   (if (< (nmpChatQueue size:) 40)
										   (self putMsg: nmpChatQueue msg)
										else
											(msg dispose:)
										)
									)
								else
									(if passGameTo
										(passGameTo handleMsg: msg)
									else
									   ;; put all other Send messages in game queue
										(self putMsg: nmpGameQueue msg)
									)
								)
							)

							(else
								(if passSystemTo
									(passSystemTo handleMsg: msg)
								else
								   ;; put all non-Send messages in system queue
								   (self putMsg: nmpSystemQueue msg)
								)
							)
						)
						(return event)
					)
				)

				(if (IsObject msg)
					(msg dispose:)
				)
			)

			(return event)
		)

		(return FALSE)
	)
)

;Estimated global vars needed: netMsgProcessor, inDialogHandler, inDialog
;Estimated classes affected: DIALOG, BUTTON, GAMECTRL
                    
