;;;;	Project: SN
;;;;
;;;;	Module: group.sc
;;;;
;;;;	Author: Jeff Stephenson
;;;;
;;;;	The Group class for network games.


(module#	GROUP)



(class Group of Set
	(properties
		id		type			0				;class type of members
		int	maximum		0				;max # of allowed members
		int	hasChanged	0				;has group changed since last polled?
		id		handler		NULLID		; Group handler
	)


	(methods
		getMembers
		addMember
		deleteMember
	)


	(method (handleMsg msg &tmp obj)
		(super handleMsg: msg)
		(= obj self)
		(switch (msg command?)
			(SN_GRPMEM
				(self getMembers: (msg members?) (msg number?))
			)
			(SN_GRPJOIN
				(= obj (self addMember: (msg who?)))
			)
			(SN_GRPDEL
				(= obj (self deleteMember: (Host lookup: (msg who?))))
			)
		)
		(if (and handler obj)
			(handler handleMsg: msg obj)
			(= hasChanged (not (msg claimed?)))
		)
		(return (msg claimed?))
	)


	(method (joinNet objNum)
		(TSN TSN_SEND
			"wwwbbww"
			SN_OBJNEW
			0
			self
			NEW_GROUP
			(theGame number?)
			objNum
			maximum
		)
	)


	(method (add whom)
		(TSN TSN_SEND
			"w+"
			SN_GRPJOIN
			sid
			(whom sid?)
		)
	)


	(method (delete whom)

		(if (and sid (self contains: whom))

			(TSN TSN_SEND
				"w+"
				SN_GRPDEL
				sid
				(whom sid?)
			)
			(return whom)
		)
		(return NULLID)
	)


	(method (getMembers mem num &tmp i)
		(self release:)
		(for	((= i 0))
				(< i num)
				((++ i))

			(self addMember: (mem at: i))
		)
		(mem dispose:)
	)


	(method (addMember who &tmp obj)
		(= obj 0)
		(if (== (= obj (Host lookup: who)) NULLID)
			(if (IsObject type)
				((= obj (type new:))
					setSid: who,
					init:,
				)
			)
		)
		(if obj
			(super add: obj)
			(= hasChanged TRUE)
		)
		(return obj)
	)


	(method (deleteMember mem)
		(if mem
			(super delete: mem)
			(= hasChanged TRUE)
		)
		(return mem)
	)


	(method (dispose)
		(if (self isEmpty:)
			(super dispose:)
		else
			(Printf "Err: Attempt to dispose of net group which is not empty.")
			(if (Debug)
				(SetDebug self)
			)
			;
			;Leaving these objects around because maybe some other stuff
			;is using them. I fear breaking other modules by disposing
			;of the elements now.
			;
			(self eachElementDo: #sid NULLID,
					release:,
			)
			(super dispose:)
		)
		(= handler NULLID)
	)
)

