(module# MSGPROC)

(define CHAT 			1)

(include globals.sh)
(include tsn.sh)

(class NetMsgProcessor of Code
   (properties
	   id nmpSystemQueue 0		;; system message queue **highest priority**
		id nmpGameQueue   0     ;; game message queue **high priority**
		id nmpChatQueue   0     ;; chat message queue **lowest prioirty**

		id lastProcessor 0   ;; last NetMsgProcessor
		id lastMsg		  0   ;; last message that was processed
		
		id	passSystemTo	0	;; ID of object to pass system messages to
		id passChatTo		0 	;; ID of object to pass chat messages to
		id passGameTo 0		;; ID of object to pass game messages to
	)

	(methods
	   init
		dispose

		getMsg		;; retrieve a message from a queue
		putMsg		;; put a message in a queue

		handleMsg	;; process a message from a given queue
		handleEvent ;; takes a remote event messages and stuffs them into appropriate queues

		doit			;; process the queues or stuff them if necessary
	)

	(method (init)
	   (super init:)

		;; if the global NetMessageProcessor is not this NetMessageProcessor
		;; then attach this one.
		(if (!= netMsgProcessor self)
			(= lastProcessor netMsgProcessor)

		   ;; attach myself to the global netMsgProcessor and inDialogHandler
			;; variables
		   (= netMsgProcessor (= inDialogHandler self))

			(= nmpSystemQueue (Set new:))
		)
	)

	(method (dispose)
	   (= netMsgProcessor (= inDialogHandler lastProcessor))

		(nmpSystemQueue dispose:)
		(= nmpSystemQueue 0)

		(super dispose:)
	)

	;** method getMsg: will return the message you request
	(method (getMsg whichMsg &tmp msg)
	   ;; get the passed message from the passed queue unless there is no 
		;; passed message number, then get message number zero.
	   (= msg (nmpSystemQueue at: (if (> argc 1) whichMsg else 0)))

		;; delete the message that we just got, the update the lastMsg 
		(nmpSystemQueue delete: (= lastMsg msg))
		(return msg)
	)

	(method (putMsg msg) ;; &tmp s oldport)
	   (nmpSystemQueue add: msg)
	)

	(method (handleMsg &tmp msg who)
	   ;; process message from the passed message queue
		(= msg (self getMsg: &rest))

		(if 
			(or
				(not (IsObject msg?))
				(not (msg isKindOf: Msg))
			)

			(return FALSE)
		)

		;; make sure that messages have the needed 'from' property set
		(if 
			(and
				(== (msg command?) SN_SEND)
				(msg respondsTo: #from)
				(msg respondsTo: #fromSID)
			)

			(msg from: (Host lookup: (msg fromSID?)))
		)

		(if (and
				(msg toSID?)
				(IsObject (= who (Host lookup: (msg toSID?))))
			)
			(msg to: who)
		)

		(if (IsObject (msg to?))
		   ;; whoever this message is for will handle it 
		   ((msg to?) handleMsg: msg)
		else
		   (if debugOn
		   	(Print "NetMsgProcessor: Msg with illegal to? property.")
			)
		)
		;; only dispose of the message if it is not already in the queues
		(if (and 
				(IsObject self) ;;; KEEP IN -- very important
				(IsObject msg) 
				(not (nmpSystemQueue contains: msg))
			)
			(msg dispose:)
		)

		;; TSN_DOIT and TSN_FLUSH if necessary
		(if (and inDialog (>> (TSN TSN_INFO) 8))
		   (TSN TSN_DOIT)
			(TSN TSN_FLUSH)
		)

		(return FALSE)
	)

	(method (doit &tmp msg obj)
	   (cond
			;; if in stasis state do not process any other messages.
			(inDialog
				(return (self handleEvent:))
			)					   

			;; process the queues if we are not in a dialog
			(else
				(return (self handleMsg:))
			)
		)
	)

	(method (handleEvent evt &tmp event msg s s1)
	   (if (>> (TSN TSN_INFO) 8)
			(TSN TSN_FLUSH)
			(TSN TSN_DOIT)

		   ;; if argument is passed in, then do a normal handleEvent, otherwise
			;; caller is expecting this function to get the remote event.
			(= event
			   (if (== argc 1)
				   evt
				else
				   (user nextEvent: remoteEvt)
				)
			)

			(if 
			   (and
			      (IsObject event)
					(== (event type?) remoteEvt)
				)

				(= msg (Msg with: (evt message?) (event modifiers?)))

				(if (== msg NULLID)
				   (return FALSE)
				)

				(if (& debugMsgs SHOW_MSG)
				   (Debug DInspect msg)
				)

				(cond
				   ((IsObject (msg to?))
					   (cond
						   ((== (msg command?) SN_SEND)
							   ;; put all chat messages in chat queue (max 40)
								(if (== (msg msgType?) CHAT) 
									(if passChatTo
										(passChatTo handleMsg: msg)

										;; if the passChatTo did not use the message
										;; put it in the queue again
										(if (not (msg claimed?))
											(self putMsg: msg)
										)
									else
									   (self putMsg: msg)
									)
								else
									(if passGameTo
										(passGameTo handleMsg: msg)

										;; if the passGameTo did not use the message
										;; put it in the queue again
										(if (not (msg claimed?))
											(self putMsg: msg)
										)
									else
									   ;; put all other Send messages in game queue
										(self putMsg: msg)
									)
								)
							)

							(else
								(if passSystemTo
									(passSystemTo handleMsg: msg)

									;; if the passSystemTo did not use the message
									;; put it in the queue again
									(if (not (msg claimed?))
										(self putMsg: msg)
									)
								else
								   ;; put all non-Send messages in system queue
								   (self putMsg: msg)
								)
							)
						)
						(return event)
					)
				)

				(if (IsObject msg)
					(msg dispose:)
				)
			)

			(return event)
		)

		(return FALSE)
	)
)

