;********************************************************************
;***
;*** PACKDATA.SC: Packed data array 
;***
;*** Takes out all the mess of packing a bunch of numbers/arrays
;*** into another array. Just need to do put/putArray and then 
;*** get/getArray.
;***
;*** Last Updated: November 6, 1990
;***
;********************************************************************


(module# PACKDATA)

(define T_PACKED_DATA	255)

(class PackedData of Array
	(properties
		int  index: 0
		int  type:	BYTEARRAY
	)
	(methods
		get
		getWord
		getArray
		put
		putWord
		putArray
	)
	(method (new)
		(= index 0)
		(super new: &rest)
	)
	(method (dispose)
		(= index 0)
		(super dispose: &rest)
	)
	;;
	;;	Bail out 0 if application is being stupid
	;; Let the bungler figure out why the data disappears
	;;
	(method (get &tmp ret)
		(if (< index 0)
			(return 0)
		else
			(= ret (self at: index))
			(++ index)
			(return ret)
		)
	)
	;;
	;;	Bail out 0 if application is being stupid
	;; Let the bungler figure out why the data disappears
	;;
	(method (getWord &tmp ret)
		(if (< index 0)
			(return 0)
		else
			(= ret (| (self get:) (<< (self get:) 8)))
			(return ret)
		)
	)
	(method (getArray &tmp obj arraySiz arrayType i)
		(= arraySiz (self getWord:))
	;;
	;;	Bail if application is being stupid
	;; Let the bungler figure out why the data disappears
	;; If he's not checking for a return value, PMACHINE!
	;;
		(if
			(or
				(< arraySiz 0)
				(< (+ arraySiz index) 0)
			)
			(return NULLID)
		)
		(= arrayType (self get:))
		(switch arrayType
			(WORDARRAY
				(= obj (IntArray new: arraySiz))
			)
			(BYTEARRAY
				(= obj (ByteArray new: arraySiz))
			)
			(STRARRAY
				(= obj (String new: arraySiz))
			)
			(else
				(= obj (PackedData new: arraySiz))
			)
		)
		(if (== arrayType WORDARRAY)
			(for ((= i 0)) (< i arraySiz) ((++ i))
				(obj at: i (self getWord:))
			)
		else
			(obj copyToFrom: 0 self (self index?) arraySiz)
			(+= index arraySiz)
		;; Ensure NUL terminated
			(if (== arrayType STRARRAY)
				(obj at: arraySiz 0)
			)
		)
		(return obj)
	)
	;;
	;;	Bail if application is being stupid
	;; Let the bungler figure out why the data disappears
	;;
	(method (put args &tmp i)
		(for ((= i 0)) (and (< i argc) (> index -1)) ((++ i))
			(self at: index [args i])
			(++ index)
		)
	)
	;;
	;;	Bail if application is being stupid
	;; Let the bungler figure out why the data disappears
	;;
	(method (putWord args &tmp i)
		(for ((= i 0)) (and (< i argc) (> index -1)) ((++ i))
			(self
				put:
					(& [args i] $FF)
					(/ [args i] 256)	;deliberately broken logic shift
			)
		)
	)
	(method (putArray array arrSize &tmp arraySize arrayType i)
		(if (> argc 1)
			(= arraySize arrSize)
		else
			(= arraySize (array size:))
		)
	;;
	;;	Bail if application is being stupid
	;; Let the bungler figure out why the data disappears
	;; If he's not checking for a return value
	;;
		(if
			(or
				(< arraySize 0)
				(< (+ arraySize index) 0)
			)
			(return FALSE)
		)
		
		(= arrayType
			(if (array isKindOf: PackedData)
				T_PACKED_DATA
			else
				(array type?)
			)
		)
		(self
			putWord:	arraySize,
			put:		arrayType,
		)
		(if (== arrayType WORDARRAY)
			(for ((= i 0)) (< i arraySize) ((++ i))
				(self putWord: (array at: i))
			)
		else
			(self copyToFrom: index array 0 arraySize)
			(+= index arraySize)
		)
		(return TRUE)
	)
)

