;;;;
;;;;	TSN.SH -- intermodule communications definitions for Sierra Network
;;;;
;;;;	Author: Jeff Stephenson


(enum 59950
	HOST
	MSG
	NETUSER
	PLAYER
	GROUP
	PACKDATA
	MSGQUEUE
	MSGPROC
)


;;; Communication commands.
(enum
	SN_ACK			;; 0
	SN_NAK
	SN_SEND
	SN_RSEND
	SN_LOCK
	SN_LOCKQUEUE
	SN_UNLOCK
	SN_OBJNEW
	SN_OBJID
	SN_OBJFREE

	SN_GRPJOIN		;; 10
	SN_GRPDEL
	SN_GRPMEM
	SN_SETINT
	SN_SETSTR
	SN_SETBYTEARRAY
	SN_SETINTARRAY
	SN_GETPROP
	SN_GETOBJ
	SN_ADD

	SN_SETEQ			;; 20
	SN_SETNE
	SN_LOGIN			;; this is what always comes back from other SN_LOGINs
	SN_GAMERIGHTS
	SN_GAMEKILL
	SN_RMI
	SN_SETUSERSID
	SN_BBS
	SN_SENDMANY
	SN_GAMEVERSION

	SN_REGISTER		;; 30
	SN_GRPPROP
	SN_EGETPROP
	SN_SET
	SN_ACCOUNTING
	SN_XFER
	SN_HOSTINFO
	SN_EMAIL
	SN_HINT
	SN_HOSTFILE

	SN_USERINFO		;; 40
	SN_OBJEXISTS
	SERV_LOGIN 		;; Server commands
	SERV_LOGOUT		;;
	SN_PASSWORD
	US_REQUEST
	SN_LOGIN_PRE_RELEASE ;; This is for PRE-release version software
	SN_WAITGRPREQUEST
	SN_UNSOLICITED
	SN_USERFIND

	SN_MUTEUNMUTE		;; 50
	SN_OBJNEWLIST
	SN_OBJFREELIST
	SN_LOGINPWVER		;; 53 This is for RELEASE version software
   SN_PERMANENT_SESSION
   SN_OL_SIGNUP
   SN_REMOTESTATUS
   SN_SETINTGRP
   SN_SETSTRGRP
	SN_LOGIN_PRODIGY	;; 59 This is for Prodigy user login

	65
	LDBS_COMMAND
	SN_SYSOP_SYSTEM  ;; New sysop commands

	146
	SN_SYSOP_LOGIN
	SN_SYSOP_CMD
	SN_DEBUG
)

;;
;;	User search types
;;
(define UF_FIND_BY_UID	1)
(define UF_FIND_BY_NAME 2)

;;
;;	Assign these to an array in msg.sc for debugging
;;
(define MsgNames
	"SN_ACK"
	"SN_NAK"
	"SN_SEND"
	"SN_RSEND"
	"SN_LOCK"
	"SN_LOCKQUEUE"
	"SN_UNLOCK"
	"SN_OBJNEW"
	"SN_OBJID"
	"SN_OBJFREE"

	"SN_GRPJOIN"
	"SN_GRPDEL"
	"SN_GRPMEM"
	"SN_SETINT"
	"SN_SETSTR"
	"SN_SETBYTEARRAY"
	"SN_SETINTARRAY"
	"SN_GETPROP"
	"SN_GETOBJ"
	"SN_ADD"

	"SN_SETEQ"
	"SN_SETNE"
	"SN_LOGIN"
	"SN_GAMERIGHTS"
	"SN_GAMEKILL"
	"SN_RMI"
	"SN_SETUSERSID"
	"SN_BBS"
	"SN_SENDMANY"
	"SN_GAMEVERSION"

	"SN_REGISTER"
	"SN_GRPPROP"
	"SN_EGETPROP"
	"SN_SET"
	"SN_ACCOUNTING"
	"SN_XFER"
	"SN_HOSTINFO"
	"SN_EMAIL"
	"SN_HINT"
	"SN_HOSTFILE"

	"SN_USERINFO"
	"SN_OBJEXISTS"
	"SERV_LOGIN"
	"SERV_LOGOUT"
	"SN_PASSWORD"
	"US_REQUEST"
	"SN_LOGIN_PRE_RELEASE"
	"SN_WAITGRPREQUEST"
	"SN_UNSOLICITED"
	"SN_USERFIND"

	"SN_MUTEUNMUTE"
	"SN_OBJNEWLIST"
	"SN_OBJFREELIST"
	"SN_LOGINPWVER"
	"SN_TRIVIAL"
	"SN_OL_SIGNUP"
	"SN_REMOTESTATUS"
	"SN_SETINTGRP"
	"SN_SETSTRGRP"
	"SN_LOGIN_PRODIGY"
)


(define	NEW_OBJECT		1)
(define	NEW_GROUP		2)
(define	FIXED_OBJECT	3)
(define	FIXED_GROUP		4)
(define	NEW_LAND			5)

(define	PERMANENT		$80)

(define	ID_UNIQUE		-1)
(define	ID_SHARED		-2)


;;; Communications errors
(enum
	ERR_NONE
	ERR_CONNECT_LOST
	ERR_NAK_EXCEEDED
	ERR_NAK_TIMEOUT
	ERR_PACKET_SIZE_EXCEEDED
	ERR_NO_DSR
	ERR_STUCK_CD
	ERR_MODEM_RESPONSE
	ERR_LINE_BUSY
	ERR_NO_ANSWER					;9

	ERR_NO_CARRIER
	ERR_NO_PACKETS
	ERR_NO_TELENET_PROMPT
	ERR_NO_TELENET_CONNECT
	ERR_NO_CONNECTION
	ERR_NAME_SHORT
	ERR_NAME_LONG
	ERR_NO_STATION
	ERR_STATION_NO_RESPONSE
	ERR_IPX_SETUP					;19

	ERR_RECONNECT_UNSUPPORTED
	ERR_NO_RECONNECT_PROMPT
	ERR_TELENET_FALLBACK
	ERR_KB_BREAKOUT				;23
	ERR_24

	ERR_EASY_MISSING				;25 EasyLink errs
	ERR_EASY_NO_RESPOND
	ERR_EASY_DEST_UNKNOWN
	ERR_EASY_NO_BIN_MODE
	ERR_EASY_WRONG_DRIVER		;29

	ERR_EASY_TIME_OUT
	ERR_EASY_CHANGE_BAUD
	ERR_SPRINT_CHANGE_BAUD
	ERR_MODEM_WRONG_DRIVER
	ERR_MODEM_RING_NO_ANSWER
	ERR_EASY_CFG_FILE_OPEN
	ERR_EASY_NO_UID
	ERR_EASY_NO_HOST_ID
	ERR_EASY_NO_PREFIX
	ERR_EASY_NO_PHONE				;39

	ERR_WRONG_COUNT
	ERR_WRONG_INTERRUPT
)


;;; Debug stuff for communications.

(define	RCV_NOT_OBJ		%00000001)
(define	SHOW_MSG			%00000010)
(define	BRK_ON_MSG		%00000100)


;;; Notification defines.
(enum
	NOTIFY_NOBODY
	NOTIFY_MYSELF
	NOTIFY_CLONES
	NOTIFY_ONLYCLONES
	NOTIFY_CURGROUP
	NOTIFY_GROUP
)


;;; Host info defines
(enum
	HOST_NAME
	HOST_ADDRESSES
	HOST_TIME
	HOST_STATS
	HOST_WAITROOMS
	HOST_NUMBER
	HOST_WAITROOMS_NEW
	HOST_LAND_CFG
	HOST_GAMEINFO
)

(define	OLD_US_EMAILREQUEST	0)
(define	US_EMAILREQUEST		1)

(define	MYPASSWORD     	1)
(define	CHANGEPASSWORD		2)

(enum 1
	YOUR_BOX_IS
	HOST_CHOKED
	DATABASE_CHOKED
	DATABASE_BUSY
	DATABASE_DEAD
	UID_SERVER_DEAD
)

;;; LOGIN NAK SUBCMDS
(enum
	LOGIN_BAD_ID
	LOGIN_NO_FREE
	LOGIN_MULTIPLE
	LOGIN_NOT_ALLOWED
	LOGIN_HOST_BUSY
	LOGIN_OCCUPIED
	LOGIN_NOT_FLAT
	LOGIN_NOT_PRIME
	LOGIN_BAD_TSN
	PASSWORD_FAIL_WRONG

	PASSWORD_FAIL_STRUCTURE
	PASSWORD_OUT_OF_DATE
	USER_LOCKOUT
	OK_GO_PLAY
	LOGIN_FAIL_OUT_OF_CREDITS
	BILLING_PROBLEMS
	PASSWORD_NOT_CHANGED
	LOGIN_VER_TOO_HIGH
	LOGIN_VER_TOO_LOW
	LOGIN_VER_LOCKOUT

	LOGIN_BAD_GAMENUM
)

;; Accounting messages
(enum
	ACCOUNT_READ_LOG
	ACCOUNT_DEL_LOG
	ACCOUNT_DEL_LOG1
	ACCOUNT_REN_LOG
	ACCOUNT_SEND_STATUS
	ACCOUNT_GET_STATUS						;; 5
	ACCOUNT_SEND_STATUS_ADMIN   			;; 6 used by admin program only
	ACCOUNT_SEND_ACCOUNT_STATUS_ADMIN   ;; 7 used by admin program only
	ACCOUNT_ACKNOWLEDGE_STATUS

	;; Sessions? Gawd I hope it's just a summary!
   ACC_SUMMARY
   QUEUE_IT
   CANCEL_QUEUE_IT
   SUMM_GET_IT_FIRST
   SUMM_GET_IT_NEXT
   SUMM_GET_IT_FINISHED
   ACC_BILLING
)

;; NAK subCmds on GRPJOIN
(enum 1
   GRPJOIN_NAK_INVALID_GROUP
   GRPJOIN_NAK_FULL
   GRPJOIN_NAK_INVALID_OBJECT
   GRPJOIN_NAK_LOCKED
   GRPJOIN_NAK_NO_LAND_RIGHTS
   GRPJOIN_NAK_VERSION
)

;; subCmds for UserInfo
(enum 1
	UI_RECV_MSG
	UI_PUT
	UI_GET
	UI_SET_USERNAME
)

;; Account summary NAKs
(enum 1
   NOT_YOUR_UID
   INVALID_UID
   SERVER_IS_DOWN
   YOU_HAVE_TRIED_TOO_MANY_TIMES
   START_DATE_OUT_OF_RANGE
   STOP_DATE_OUT_OF_RANGE
   BOTH_DATES_OUT_OF_RANGE
   ALREADY_QUEUED
   NOT_QUEUED
   TOO_BUSY_TRY_AGAIN_LATER
   NOTHING_TO_STOP
   DATA_NOT_READY
)
;; Account summary info
(enum 0
	ACC_SUMMARY_0
	ACC_SUMMARY_1
	ACC_SUMMARY_2
)

;; server information for ATM
(enum 1
	SERVER_UP
	SERVER_DOWN
)

(enum 1
	ATM_DEPOSIT
	ATM_WITHDRAW
)

